unit ufrmMain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  Buttons, ComCtrls, IniFiles, fpJson, jsonparser;

type

  TDeviceType = (dtDevice, dtContact);

  TDevice = class(TObject)
  public
    Active: Boolean;
    Created: TDateTime;
    DeviceType: TDeviceType;
    Email: String;
    Id: String;
    Iden: String;
    Kind: String;
    Manufacturer: String;
    Model: String;
    Modified: TDateTime;
    Nickname: String;
    Status: String;
    constructor Create;
    function DisplayName: string;
  end;

  { TfrmMain }

  TfrmMain = class(TForm)
    btnPush: TButton;
    btnSendFile: TButton;
    cmbDevices: TComboBox;
    lbAdvanced: TLabel;
    stbMain: TStatusBar;
    txtTitle: TEdit;
    Label1: TLabel;
    btnConfig: TSpeedButton;
    Label2: TLabel;
    Label3: TLabel;
    txtBody: TMemo;
    procedure btnConfigClick(Sender: TObject);
    procedure btnPushClick(Sender: TObject);
    procedure btnSendFileClick(Sender: TObject);
    procedure cmbDevicesChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure lbAdvancedClick(Sender: TObject);
    procedure txtBodyChange(Sender: TObject);
  private
    { private declarations }
    blUseProxy: Boolean;
    strProxyHost: String;
    intProxyPort: Integer;
    strAPIKey: String;
    procedure ReadConf;
    procedure GetDevices;
    procedure ClearDevices;
    procedure CheckResponse(Response: String);
    function Get(Url: AnsiString): String;
    function Push(Url, Title, ContentType, Content: AnsiString;
      Device: TDevice): String;
  public
    { public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.lfm}

{ TfrmMain }

uses ufrmAPIKey, ufrmAdvanced, Curl, DateUtils;

function CurlProgress(clientp: Pointer; dltotal, dlnow, ultotal,
  ulnow: Double): Integer; cdecl;
begin
  if ultotal > 0 then
  begin
    frmMain.stbMain.Panels[0].Text:= FormatFloat('0.00',(100*ulnow)/ultotal) +
      '% Upload';
    Application.ProcessMessages;;
  end else
    frmMain.stbMain.Panels[0].Text:= EmptyStr;
end;

function TfrmMain.Get(Url: AnsiString): String;
var
  Curl: TCURL;
  Response: TStringStream;
begin
  Result:= EmptyStr;
  Curl:= curl_easy_init;
  if Curl <> nil then
  try
    if (curl_easy_setopt(Curl, CURLOPT_VERBOSE, FALSE) <> CURLE_OK) or
       (curl_easy_setopt(Curl, CURLOPT_USE_SSL, CURLUSESSL_ALL) <> CURLE_OK) or
       (curl_easy_setopt(Curl, CURLOPT_SSL_VERIFYPEER, FALSE) <> CURLE_OK) or
       (curl_easy_setopt(Curl, CURLOPT_URL, PAnsiChar(URL)) <> CURLE_OK) or
       (curl_easy_setopt(Curl, CURLOPT_USERPWD, PAnsiChar(strAPIKey+':')) <> CURLE_OK) or
       (curl_easy_setopt(Curl, CURLOPT_WRITEFUNCTION, @CurlSaveToStream) <> CURLE_OK) then
      Exit;
    if blUseProxy and (strProxyHost <> EmptyStr) then
    begin
      if (curl_easy_setopt(Curl, CURLOPT_PROXY, PAnsiChar(strProxyHost)) <> CURLE_OK) then
        Exit;
      if (intProxyPort > 0) and (curl_easy_setopt(Curl, CURLOPT_PROXYPORT, intProxyPort) <> CURLE_OK) then
        Exit;
    end;
    Response:= TStringStream.Create(EmptyStr);
    try
      if curl_easy_setopt(Curl, CURLOPT_FILE, Pointer(Response)) <> CURLE_OK then
        Exit;
      if curl_easy_perform(Curl) = CURLE_OK then
        Result:= Response.DataString;
    finally
      Response.Free;
    end;
  finally
    curl_easy_cleanup(Curl);
  end;
end;

function TfrmMain.Push(Url, Title, ContentType, Content: AnsiString;
  Device: TDevice): String;
var
  Curl: TCURL;
  Response: TStringStream;
  HttpPost, Last: pcurl_httppost;
  ContentName: AnsiString;
begin
  Result:= EmptyStr;
  Curl:= curl_easy_init;
  if Curl <> nil then
  try
    if (curl_easy_setopt(Curl, CURLOPT_VERBOSE, FALSE) <> CURLE_OK) or
       (curl_easy_setopt(Curl, CURLOPT_USE_SSL, CURLUSESSL_ALL) <> CURLE_OK) or
       (curl_easy_setopt(Curl, CURLOPT_SSL_VERIFYPEER, FALSE) <> CURLE_OK) or
       (curl_easy_setopt(Curl, CURLOPT_URL, PAnsiChar(URL)) <> CURLE_OK) or
       (curl_easy_setopt(Curl, CURLOPT_USERPWD, PAnsiChar(strAPIKey+':')) <> CURLE_OK) or
       (curl_easy_setopt(Curl, CURLOPT_NOPROGRESS, 0) <> CURLE_OK) or
       (curl_easy_setopt(Curl, CURLOPT_PROGRESSDATA, nil) <> CURLE_OK) or
       (curl_easy_setopt(Curl, CURLOPT_PROGRESSFUNCTION, @CurlProgress) <> CURLE_OK) or
       (curl_easy_setopt(Curl, CURLOPT_WRITEFUNCTION, @CurlSaveToStream) <> CURLE_OK) then
      Exit;
    if blUseProxy and (strProxyHost <> EmptyStr) then
    begin
      if (curl_easy_setopt(Curl, CURLOPT_PROXY, PAnsiChar(strProxyHost)) <> CURLE_OK) then
        Exit;
      if (intProxyPort > 0) and (curl_easy_setopt(Curl, CURLOPT_PROXYPORT, intProxyPort) <> CURLE_OK) then
        Exit;
    end;
    HttpPost:= nil;
    Last:= nil;
    try
      if Device.DeviceType = dtDevice then
      begin
        if (curl_formadd(@HttpPost, @Last, CURLFORM_COPYNAME, PAnsiChar('device_iden'),
          CURLFORM_COPYCONTENTS, PAnsiChar(Device.Iden), CURLFORM_END) <> CURL_FORMADD_OK) then
          Exit;
      end else if Device.DeviceType = dtContact then
      begin
        if (curl_formadd(@HttpPost, @Last, CURLFORM_COPYNAME, PAnsiChar('target_email'),
          CURLFORM_COPYCONTENTS, PAnsiChar(Device.Email), CURLFORM_END) <> CURL_FORMADD_OK) then
          Exit;
      end else
        Exit;
      if (Title <> EmptyStr) and (curl_formadd(@HttpPost, @Last, CURLFORM_COPYNAME, PAnsiChar('title'),
          CURLFORM_COPYCONTENTS, PAnsiChar(Title), CURLFORM_END) <> CURL_FORMADD_OK) then
        Exit;
      if (curl_formadd(@HttpPost, @Last, CURLFORM_COPYNAME, PAnsiChar('type'),
          CURLFORM_COPYCONTENTS, PAnsiChar(ContentType), CURLFORM_END) <> CURL_FORMADD_OK) then
        Exit;
      ContentName:= 'body';
      if ContentType = 'note' then
        ContentName:= 'body'
      else if ContentType = 'url' then
        ContentName:= 'link'
      else if ContentType = 'address' then
        ContentName:= 'address'
      else if ContentType = 'file' then
        ContentName:= 'file';
      if ContentType = 'file' then
      begin
         if (curl_formadd(@HttpPost, @Last, CURLFORM_COPYNAME, PAnsiChar('file'),
             CURLFORM_FILE, PAnsiChar(Content), CURLFORM_END) <> CURL_FORMADD_OK) then
           Exit;
      end else
        if (curl_formadd(@HttpPost, @Last, CURLFORM_COPYNAME, PAnsiChar(ContentName),
          CURLFORM_COPYCONTENTS, PAnsiChar(Content), CURLFORM_END) <> CURL_FORMADD_OK) then
          Exit;
      Response:= TStringStream.Create(EmptyStr);
      try
        if (curl_easy_setopt(Curl, CURLOPT_FILE, Pointer(Response)) <> CURLE_OK) or
           (curl_easy_setopt(Curl, CURLOPT_HTTPPOST, HttpPost) <> CURLE_OK) then
          Exit;
        if curl_easy_perform(Curl) = CURLE_OK then
          Result:= Response.DataString;
      finally
        Response.Free;
      end;
    finally
      if HttpPost <> nil then
        curl_formfree(HttpPost);
    end;
  finally
    curl_easy_cleanup(Curl);
  end;
end;

constructor TDevice.Create;
begin
  Active:= TRUE;
  Created:= 0;
  DeviceType:= dtDevice;
  Email:= EmptyStr;
  Id:= EmptyStr;
  Iden:= EmptyStr;
  Kind:= EmptyStr;
  Manufacturer:= EmptyStr;
  Model:= EmptyStr;
  Modified:= 0;
  Nickname:= EmptyStr;
  Status:= EmptyStr;
end;

function TDevice.DisplayName: String;
begin
  if NickName <> EmptyStr then
    Result:= NickName
  else if Model <> EmptyStr then
    Result:= Model
  else if Iden <> EmptyStr then
    Result:= Iden;
  if DeviceType = dtContact then
    Result:= Result + ' (' + Email + ')';
end;

procedure TfrmMain.GetDevices;
var
  i: Integer;
  Response: String;
  Data, Devices: TJSONData;
  Device: TJSONObject;
  Item: TDevice;
begin
  ClearDevices;
  // Obtener lista de devices
  Response:= Trim(Get('https://api.pushbullet.com/v2/devices'));
  if Response = EmptyStr then
    Exit;
  with TJSONParser.Create(Response) do
  try
    try
      Data:= Parse;
      try
        if Data.JSONType = jtObject then
        begin
          Devices:= TJSONObject(Data).Find('devices');
          if (Devices <> nil) and (Devices.Count > 0) then
          begin
            for i:= 0 to Devices.Count-1 do
            begin
              Device:= TJSONObject(Devices.Items[i]);
              Item:= TDevice.Create;
              with Item do
              begin
                if Device.Find('active') <> nil then
                  Active:= Uppercase(Trim(Device.Find('active').AsString)) = 'TRUE';
                if Device.Find('created') <> nil then
                  Created:= UnixToDateTime(Trunc(StrToFloatDef(
                  StringReplace(Trim(Device.Find('created').AsString),'.',
                    DecimalSeparator,[rfReplaceAll]),0)));
                if Device.Find('email') <> nil then
                  Email:= Device.Find('email').AsString;
                if Device.Find('id') <> nil then
                  Id:= Device.Find('id').AsString
                else
                  Continue;
                if Device.Find('iden') <> nil then
                  Iden:= Device.Find('iden').AsString
                else
                  Continue;
                if Device.Find('kind') <> nil then
                  Kind:= Device.Find('kind').AsString;
                if Device.Find('manufacturer') <> nil then
                  Manufacturer:= Device.Find('manufacturer').AsString;
                if Device.Find('model') <> nil then
                  Model:= Device.Find('model').AsString;
                if Device.Find('modified') <> nil then
                  Modified:= UnixToDateTime(Trunc(StrToFloatDef(
                  StringReplace(Trim(Device.Find('modified').AsString),'.',
                    DecimalSeparator,[rfReplaceAll]),0)));
                if Device.Find('nickname') <> nil then
                  Nickname:= Device.Find('nickname').AsString;
                if Active then
                  cmbDevices.Items.AddObject(Item.DisplayName, TObject(Item));
              end;
            end;
          end;
        end;
      finally
        Data.Free;
      end;
    except
      On E: Exception do
      begin
        ClearDevices;
        Exit;
      end;
    end;
  finally
    Free;
  end;
  // Obtener lista de contactos
  Response:= Trim(Get('https://api.pushbullet.com/v2/contacts'));
  if Response = EmptyStr then
    Exit;
  with TJSONParser.Create(Response) do
  try
    try
      Data:= Parse;
      try
        if Data.JSONType = jtObject then
        begin
          Devices:= TJSONObject(Data).Find('contacts');
          if (Devices <> nil) and (Devices.Count > 0) then
          begin
            for i:= 0 to Devices.Count-1 do
            begin
              Device:= TJSONObject(Devices.Items[i]);
              Item:= TDevice.Create;
              with Item do
              begin
                DeviceType:= dtContact;
                if Device.Find('active') <> nil then
                  Active:= Uppercase(Trim(Device.Find('active').AsString)) = 'TRUE';
                if Device.Find('created') <> nil then
                  Created:= UnixToDateTime(Trunc(StrToFloatDef(
                  StringReplace(Trim(Device.Find('created').AsString),'.',
                    DecimalSeparator,[rfReplaceAll]),0)));
                if Device.Find('id') <> nil then
                  Id:= Device.Find('id').AsString
                else
                  Continue;
                if Device.Find('iden') <> nil then
                  Iden:= Device.Find('iden').AsString
                else
                  Continue;
                if Device.Find('email') <> nil then
                  Email:= Device.Find('email').AsString
                else
                  Continue;
                if Device.Find('modified') <> nil then
                  Modified:= UnixToDateTime(Trunc(StrToFloatDef(
                  StringReplace(Trim(Device.Find('modified').AsString),'.',
                    DecimalSeparator,[rfReplaceAll]),0)));
                if Device.Find('name') <> nil then
                  Nickname:= Device.Find('name').AsString;
                if Device.Find('status') <> nil then
                  Status:= Device.Find('status').AsString;
                if Active then
                  cmbDevices.Items.AddObject(Item.DisplayName, TObject(Item));
              end;
            end;
          end;
        end;
      finally
        Data.Free;
      end;
    except
      On E: Exception do
      begin
        ClearDevices;
        Exit;
      end;
    end;
  finally
    Free;
  end;
end;

procedure TfrmMain.ClearDevices;
var
  i: Integer;
begin
  for i:= cmbDevices.Items.Count - 1 downto 0 do
  begin
    cmbDevices.Items.Objects[i].Free;
    cmbDevices.Items.Delete(i);
  end;
end;

procedure TfrmMain.ReadConf;
var
  i: Integer;
  Str: String;
begin
  ClearDevices;
  with TIniFile.Create(ChangeFileExt(ParamStr(0),'.ini')) do
  try
    // Obtenemos el valor de la APIKey
    strAPIKey:= Trim(ReadString('APIKey','Value',EmptyStr));
    // Obtenemos la configuracion del Proxy
    blUseProxy:= UpperCase(Trim(ReadString('Proxy','Enabled','FALSE'))) = 'TRUE';
    if blUseProxy then
    begin
      strProxyHost:= Trim(ReadString('Proxy','Host',EmptyStr));
      intProxyPort:= StrToIntDef(Trim(ReadString('Proxy','Port',EmptyStr)),0);
    end;
    // Obtenemos la lista de dispositivos
    GetDevices;
    // Seleccionamos el que vamos a usar
    if cmbDevices.Items.Count > 0 then
    begin
      cmbDevices.ItemIndex:= 0;
      Str:= Trim(ReadString('Device','Value',EmptyStr));
      if Str <> EmptyStr then
        for i:= 0 to cmbDevices.Items.Count - 1 do
          if TDevice(cmbDevices.Items.Objects[i]).Iden = Str then
          begin
            cmbDevices.ItemIndex:= i;
            break;
          end;
      WriteString('Device','Value',
        TDevice(cmbDevices.Items.Objects[cmbDevices.ItemIndex]).Iden);
    end else
      WriteString('Device','Value',EmptyStr);
  finally
    Free;
  end;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
var
  i,j : Integer;
  Title, ContentType,
  Content: String;
  Str: String;
  Device: TDevice;
begin
  strAPIKey:= EmptyStr;
  blUseProxy:= FALSE;
  strProxyHost:= EmptyStr;
  intProxyPort:= 0;
  ReadConf;

  Title:= EmptyStr;
  ContentType:= EmptyStr;
  Content:= EmptyStr;

  Device:= nil;

  for i:= 1 to ParamCount do
    // -tTitulo
    if Copy(ParamStr(i),1,2) = '-t' then
      Title:= Copy(ParamStr(i),3,MAXINT)
    // -f<note|file>
    else if Copy(ParamStr(i),1,2) = '-f' then
      ContentType:= Copy(ParamStr(i),3,MAXINT)
    // -c<Texto|Ruta>
    else if Copy(ParamStr(i),1,2) = '-c' then
      Content:= Copy(ParamStr(i),3,MAXINT)
    // -dNickname
    else  if Copy(ParamStr(i),1,2) = '-d' then
    begin
      Str:= Copy(ParamStr(i),3,MAXINT);
      for j:= 0 to cmbDevices.Items.Count - 1 do
        if  Str = cmbDevices.Items[j] then
        begin
          Device:= TDevice(cmbDevices.Items.Objects[j]);
          break;
        end;
    end;

  if (strAPIKey <> EmptyStr) and (Device <> nil)
    and (ContentType <> EmptyStr) and (Content <> EmptyStr) then
    begin
      try
        if ContentType = 'note' then
        begin
          Push('https://api.pushbullet.com/api/pushes',
            Title, 'note', Content, Device);
        end else
        if ContentType = 'file' then
        begin
          if FileExists(Content) then
          begin
            Push('https://api.pushbullet.com/api/pushes',
              EmptyStr, 'file', Content, Device);
          end;
        end;
      except
      end;
      Application.ShowMainForm:= FALSE;
      Application.Terminate;
    end;
end;

procedure TfrmMain.lbAdvancedClick(Sender: TObject);
begin
  with TIniFile.Create(ChangeFileExt(ParamStr(0),'.ini')) do
  try
    // Obtenemos la configuracion del proxy
    blUseProxy:= UpperCase(Trim(ReadString('Proxy','Enabled','FALSE'))) = 'TRUE';
    if blUseProxy then
    begin
      strProxyHost:= Trim(ReadString('Proxy','Host',EmptyStr));
      intProxyPort:= StrToIntDef(Trim(ReadString('Proxy','Port',EmptyStr)),0);
    end;
    with TfrmAdvanced.Create(nil) do
    try
      chkUseProxy.Checked:= blUseProxy;
      txtProxyHost.Enabled:= blUseProxy;
      txtProxyPort.Enabled:= blUseProxy;
      txtProxyHost.Text:= strProxyHost;
      if intProxyPort > 0 then
        txtProxyPort.Text:= IntToStr(intProxyPort)
      else
        txtProxyPort.Text:= EmptyStr;
      if ShowModal = mrOK then
      begin
        // Guardamos la configuracion del proxy
        blUseProxy:= chkUseProxy.Checked;
        strProxyHost:= txtProxyHost.Text;
        intProxyPort:= strToIntDef(txtProxyPort.Text,0);
        if blUseProxy then
         WriteString('Proxy','Enabled','TRUE')
        else
         WriteString('Proxy','Enabled','FALSE');
        WriteString('Proxy','Host',strProxyHost);
        WriteString('Proxy','Port',intToStr(IntProxyPort));
      end;
    finally
      Free;
    end;
  finally
    Free;
  end;
end;

procedure TfrmMain.txtBodyChange(Sender: TObject);
begin
  stbMain.Panels[0].Text:= EmptyStr;
  stbMain.Panels[1].Text:= EmptyStr;
end;

procedure TfrmMain.btnConfigClick(Sender: TObject);
begin
  stbMain.Panels[0].Text:= EmptyStr;
  stbMain.Panels[1].Text:= EmptyStr;
  with TfrmAPIKey.Create(nil) do
  try
    txtAPIKey.Text:= strAPIKey;
    if (ShowModal = mrOk) and (Trim(txtAPIKey.Text) <> EmptyStr) then
    begin
      with TIniFile.Create(ChangeFileExt(ParamStr(0),'.ini')) do
      try
        strAPIKey:= txtAPIKey.Text;
        WriteString('APIKey','Value',Trim(txtAPIKey.Text));
        ReadConf;
      finally
        Free;
      end;
    end;
  finally
    Free;
  end;
end;

procedure TfrmMain.btnPushClick(Sender: TObject);
var
  Device: TDevice;
  Title: String;
  Body: String;
begin
  stbMain.Panels[0].Text:= EmptyStr;
  stbMain.Panels[1].Text:= EmptyStr;
  Application.ProcessMessages;
  if (cmbDevices.ItemIndex >= 0) then
  begin
    Device:= TDevice(cmbDevices.Items.Objects[cmbDevices.ItemIndex]);
    Title:=    Trim(txtTitle.Text);
    Body:=     Trim(txtBody.Text);
    if (Device <> nil) and (Title <> EmptyStr) and (Body <> EmptyStr) then
    begin
      stbMain.Panels[1].Text:= 'Pushing note ...';
      CheckResponse(Push('https://api.pushbullet.com/api/pushes',
        Title, 'note', Body, Device));
    end;
  end;
end;

procedure TfrmMain.btnSendFileClick(Sender: TObject);
var
  Device: TDevice;
begin
  stbMain.Panels[0].Text:= EmptyStr;
  stbMain.Panels[1].Text:= EmptyStr;
  Application.ProcessMessages;
  if (cmbDevices.ItemIndex >= 0) then
  begin
    Device:= TDevice(cmbDevices.Items.Objects[cmbDevices.ItemIndex]);
    if (Device <> nil) then
      with TOpenDialog.Create(nil) do
      try
        if Execute then
        begin
          stbMain.Panels[1].Text:= 'Pushing file ...';
          CheckResponse(Push('https://api.pushbullet.com/api/pushes',
            EmptyStr, 'file', Filename, Device));
        end;
      finally
        Free;
      end;
  end;
end;

procedure TfrmMain.cmbDevicesChange(Sender: TObject);
begin
  stbMain.Panels[0].Text:= EmptyStr;
  stbMain.Panels[1].Text:= EmptyStr;
  with TIniFile.Create(ChangeFileExt(ParamStr(0),'.ini')) do
  try
    if cmbDevices.ItemIndex >= 0 then
      WriteString('Device','Value',
        TDevice(cmbDevices.Items.Objects[cmbDevices.ItemIndex]).Iden)
    else
      WriteString('Device','Value',EmptyStr);
  finally
    Free;
  end
end;

procedure TfrmMain.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  cmbDevicesChange(nil);
  ClearDevices;
end;

procedure TfrmMain.CheckResponse(Response: String);
var
  Data, Error: TJSONData;
begin
  Response:= Trim(Response);
  if Response = EmptyStr then
  begin
    stbMain.Panels[1].Text:= 'Message not sent';
    Exit;
  end;
  with TJSONParser.Create(Response) do
  try
    try
      Data:= Parse;
      try
        if Data.JSONType = jtObject then
        begin
          Error:= TJSONObject(Data).Find('error');
          if (Error <> nil) then
          begin
            stbMain.Panels[1].Text:= 'Error: ' +
              TJSONObject(Error).Find('type').AsString;
          end else
            stbMain.Panels[1].Text:= 'Message sent';
        end;
      finally
        Data.Free;
      end;
    except
      On E: Exception do
      begin
        stbMain.Panels[1].Text:= 'Message not sent';
        Exit;
      end;
    end;
  finally
    Free;
  end;
end;

end.

